/* Copyright (c) 2000 Michael Claen <mclassen@internet.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id$
 */

package com.exploringxml.rss;

import java.util.Vector;
import java.net.URL;
import java.io.InputStreamReader;

import com.exploringxml.xml.Xparse;
import com.exploringxml.xml.Node;

/**
 * Reads in a Rich Site Summary (RSS) file and populates its members
 * with the parsed content of the file
 * <p>does not depend on external XML parser anymore
 *
 * @author    Michael Claen
 * @version   $Revision: 1.2 $
 */
public class RSSChannel {

  private String channelTitle;
  private URL channelLink;
  private String channelDescription;
  private String imageTitle;
  private URL imageLink;
  private URL imageURL;
  private Vector items;

  private Node root;

  /**
   * loads an RSS file from a URL
   *
   * @param     srcURL the URL to load from
   * @exception Exception mostly @see java.io.IOException or @see java.net.MalformedURLException
   */
  public void load(String srcURL) throws Exception {
    root = new Xparse().parse(readChannel(srcURL));
    channelTitle = root.find("RDF/channel/title", 1).getCharacters();
    channelLink = new URL(root.find("RDF/channel/link", 1).getCharacters());
    channelDescription = root.find("RDF/channel/description", 1).getCharacters();
    imageTitle = root.find("RDF/image/title", 1).getCharacters();
    imageLink = new URL(root.find("RDF/image/link", 1).getCharacters());
    imageURL = new URL(root.find("RDF/image/url", 1).getCharacters());
    items = new Vector();
    int pos = 0;
    while (true) {
      Node n = root.find("RDF/item/title", pos+1);
      if (n == null) break;
      items.insertElementAt(n.getCharacters(), 2*pos);
      n = root.find("RDF/item/link", pos+1);
      if (n == null) break;
      items.insertElementAt(new URL(n.getCharacters()), 2*pos+1);
      ++pos;
    }
  }

  private String readChannel(String srcURL) throws Exception {
    URL u = new URL(srcURL);
    InputStreamReader r = new InputStreamReader(u.openStream());
    StringBuffer sb = new StringBuffer();
    int c;
    while ((c = r.read()) != -1) {
      sb.append((char)c);
    }
    return sb.toString();
  }

  /**
   * gets the title of the RSS channel
   *
   * @return    String denoting the RSS channel's title
   */
  public String getChannelTitle() {
    return channelTitle;
  }

  /**
   * gets the channel's associated URL
   *
   * @return    URL denoting the channel's link
   */
  public URL getChannelLink() {
    return channelLink;
  }

  /**
   * gets the channel's associated description
   *
   * @return    String denoting the channel's description
   */
  public String getChannelDescription() {
    return channelDescription;
  }

  /**
   * gets the title of the channel's associated image
   *
   * @return    String denoting the channel image's title
   */
  public String getImageTitle() {
    return imageTitle;
  }

  /**
   * gets the target URL of the channel's associated image
   *
   * @return    URL denoting the channel image's target
   */
  public URL getImageLink() {
    return imageLink;
  }

  /**
   * gets the source URL of the channel's associated image
   *
   * @return    URL denoting the channel image's source
   */
  public URL getImageURL() {
    return imageURL;
  }

  /**
   * gets the title of the referenced channel item
   *
   * @param     item the item in the channel list
   * @return    String denoting the channel item's title
   */
  public String getItemTitle(int pos) {
    return (String) items.elementAt(2*pos);
  }

  /**
   * gets the associated link of the referenced channel item
   *
   * @param     item the item in the channel list
   * @return    URL denoting the channel item's associated link
   */
  public URL getItemLink(int pos) {
    return (URL) items.elementAt(2*pos+1);
  }

  /**
   * gets the number of items in the channel
   *
   * @return    the number of items
   */
  public int getNumberOfItems() {
    return items.size() / 2;
  }

}
