/* Copyright (c) 2000 Michael Claen <mclassen@internet.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id$
 */

package com.exploringxml.rss.applet;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.net.URL;

import com.exploringxml.rss.RSSChannel;

/**
 * Reads and displays Rich Site Summary (RSS) files;
 * highly customizable look via applet parameters
 * <p>currently depends on the <a href="http://www.microstar.com/Aelfred">Aelfred</a> XML parser
 *
 * @author    Michael Claen
 * @version   $Revision$
 */
public class RSSViewerApplet extends Applet {

  private static final int TITLE_IS_CURRENT_ITEM = -1;
  private static final int NO_CURRENT_ITEM = -2;
  private int currentItem = NO_CURRENT_ITEM;
  private RSSChannel channel;
  private Exception initException;
  private Font titleFont;
  private Font itemFont;
  private int titleLineBase;
  private int titleLineHeight;
  private int itemLineBase;
  private int itemLineHeight;

  // applet params

  private String rssFileSrcURL;
  private Color boxForeground;
  private Color boxBackground;
  private int boxBorder;
  private boolean boxScroll;
  private boolean boxScrollHorizontal;
  private boolean boxScrollReverse;
  private Color titleForeground;
  private Color titleBackground;
  private Color titleForegroundOnMouseOver;
  private Color titleBackgroundOnMouseOver;
  private String titleFontFamily;
  private int titleFontSize;
  private int titleFontStyle;
  private String titleAlign;
  private Color itemForeground;
  private Color itemBackground;
  private Color itemForegroundOnMouseOver;
  private Color itemBackgroundOnMouseOver;
  private String itemFontFamily;
  private int itemFontSize;
  private int itemFontStyle;
  private String itemAlign;
  private int itemMargin;
  private String itemSeparator;

  // parameter helpers

  /**
   * gets a String parameter from the applet's context
   *
   * @param     key a String denoting the parameter's name
   * @param     defaultValue a String for parameters default value if key absent
   * @return    String denoting parameter's value, either retrieved or default
   */
  private String getParameter(String key, String defaultValue) {
    String paramValue = getParameter(key);
    return (paramValue != null ? paramValue : defaultValue);
  }

  /**
   * gets an int parameter from the applet's context
   *
   * @param     key a String denoting the parameter's name
   * @param     defaultValue an int for parameters default value if key absent
   * @return    int denoting parameter's value, either retrieved or default
   */
  private int getParameter(String key, int defaultValue) {
    String paramValue = getParameter(key);
    return (paramValue != null ? Integer.parseInt(paramValue) : defaultValue);
  }

  /**
   * gets a boolean parameter from the applet's context
   *
   * @param     key a String denoting the parameter's name
   * @param     defaultValue a boolean for parameters default value if key absent
   * @return    boolean denoting parameter's value, either retrieved or default
   */
  private boolean getParameter(String key, boolean defaultValue) {
    String paramValue = getParameter(key);
    return (paramValue != null ? paramValue.equalsIgnoreCase("true") ||
      paramValue.equalsIgnoreCase("yes") : defaultValue);
  }

  /**
   * gets a Color parameter from the applet's context
   *
   * @param     key a String denoting the parameter's name
   * @param     defaultValue a Color for parameters default value if key absent
   * @return    Color denoting parameter's value, either retrieved or default
   */
  private Color getParameter(String key, Color defaultValue) {
    String paramValue = getParameter(key);
    //TODO: improve parsing and error checking
    //      expects #RRGGBB and lacks support for predefined colors MC20000315
    return (paramValue != null ?
      new Color(Integer.parseInt(paramValue.substring(1), 16)) : defaultValue);
  }

  /**
   * gets a Font Style parameter from the applet's context
   *
   * @param     key a String denoting the parameter's name
   * @param     defaultValue a String for parameters default font style if key absent
   * @return    int denoting parameter's font style, either retrieved or default
   */
  private int getFontStyleParameter(String key, String defaultValue) {
    String paramValue = getParameter(key);
    if (paramValue == null) paramValue = defaultValue;
    if (paramValue.equalsIgnoreCase("bold")) return Font.BOLD;
    else if (paramValue.equalsIgnoreCase("italic")) return Font.ITALIC;
    else if (paramValue.equalsIgnoreCase("bolditalic")) return Font.BOLD + Font.ITALIC;
    return Font.PLAIN;
  }

  /**
   * initializes applet's parameters by retrieving <param> tags or setting defaults
   */
  private void initParameters() {
    Color darkPurple = new Color(208, 51, 255);
    Color lightPurple = new Color(224, 170, 255);
    rssFileSrcURL = getParameter("src", "http://localhost/xml/index.rss");
    boxForeground = getParameter("box.foreground", Color.white);
    boxBackground = getParameter("box.background", lightPurple);
    boxBorder = getParameter("box.border", 3);
    boxScroll = getParameter("box.scroll", false);
    boxScrollHorizontal = getParameter("box.scroll.horizontal", false);
    boxScrollReverse = getParameter("box.scroll.reverse", false);
    titleForeground = getParameter("title.foreground", Color.white);
    titleBackground = getParameter("title.background", lightPurple);
    titleForegroundOnMouseOver = getParameter("title.foreground.mouseover", darkPurple);
    titleBackgroundOnMouseOver = getParameter("title.background.mouseover", lightPurple);
    titleFontFamily = getParameter("title.font.family", "sansserif");
    titleFontSize = getParameter("title.font.size", 16);
    titleFontStyle = getFontStyleParameter("title.font.style", "bold");
    titleAlign = getParameter("title.align", "center");
    itemForeground = getParameter("item.foreground", Color.white);
    itemBackground = getParameter("item.background", darkPurple);
    itemForegroundOnMouseOver = getParameter("item.foreground.mouseover", lightPurple);
    itemBackgroundOnMouseOver = getParameter("item.background.mouseover", darkPurple);
    itemFontFamily = getParameter("item.font.family", "sansserif");
    itemFontSize = getParameter("item.font.size", 12);
    itemFontStyle = getFontStyleParameter("item.font.style", "plain");
    itemAlign = getParameter("item.align", "left");
    itemMargin = getParameter("item.margin", 1);
    itemSeparator = getParameter("item.separator", "*");
  }

  // applet lifecycle methods

  /**
   * initializes applet
   * @see java.applet.Applet#init
   */
  public void init() {
    initParameters();
    channel = new RSSChannel();
    try {
      channel.load(rssFileSrcURL);
    }
    catch (Exception ex) {
      initException = ex;
    }
  }

  /**
   * starts applet
   * @see java.applet.Applet#start
   */
  public void start() {
    titleFont = new Font(titleFontFamily, titleFontStyle, titleFontSize);
    titleLineBase = getLineBase(titleFont);
    titleLineHeight = getLineHeight(titleFont);
    itemFont = new Font(itemFontFamily, itemFontStyle, itemFontSize);
    itemLineBase = getLineBase(itemFont);
    itemLineHeight = getLineHeight(itemFont);
    // for IE for MAC who calls paint before start MC20000331
    repaint();
  }

  /**
   * stops applet
   * @see java.applet.Applet#stop
   */
  public void stop() {
    titleFont = null;
    itemFont = null;
  }

  /**
   * destroys applet
   * @see java.applet.Applet#destroy
   */
  public void destroy() {
    channel = null;
    // TODO if memory leaks in browser: destroyParameters(); MC20000315
  }

  /**
   * gets applet's info
   * @see java.applet.Applet#getAppletInfo
   */
  public String getAppletInfo() { return "RSS Viewer Applet, (c) 2000 Michael Claen"; }

  /**
   * gets applet's parameter info
   * @see java.applet.Applet#getParameterInfo
   */
  public String[][] getParameterInfo() {
    String[][] pinfo = {
      {"src", "String", "RSS file source URL"},
      {"box.foreground", "int", "box foreground color"},
      {"box.background", "int", "box background color"},
      {"box.border", "int", "box border width"},
      {"box.scroll", "boolean", "box scrolling"},
      {"box.scroll.horizontal", "boolean", "horizontal/vertical scrolling"},
      {"box.scroll.reverse", "boolean", "forward/backward scrolling"},
      {"title.foreground", "int", "title foreground color"},
      {"title.background", "int", "title background color"},
      {"title.foreground.mouseover", "int", "title foreground color on mouse-over"},
      {"title.background.mouseover", "int", "title background color on mouse-over"},
      {"title.font.family", "String", "title font family"},
      {"title.font.size", "int", "title font size"},
      {"title.font.style", "String", "title font style"},
      {"item.foreground", "int", "item foreground color"},
      {"item.background", "int", "item background color"},
      {"item.foreground.mouseover", "int", "item foreground color on mouse-over"},
      {"item.background.mouseover", "int", "item background color on mouse-over"},
      {"item.font.family", "String", "item font family"},
      {"item.font.size", "int", "item font size"},
      {"item.font.style", "String", "item font style"},
      {"item.margin", "int", "item margin"},
      {"item.separator", "String", "item separator"},
    };
    return pinfo;
  }

  // paint methods

  /**
   * paints applet
   * @see java.applet.Applet#paint
   */
  public void paint(Graphics g) {
    if (initException != null) {
      g.drawString("Error loading RSS file:", 0, 10);
      g.drawString(initException.getMessage(), 0, 20);
      return;
    }
    Rectangle bounds = new Rectangle(size().width, size().height);
    paintBox(g, bounds);
    paintTitle(g, bounds);
    narrowBounds(bounds, itemMargin, 0);
    for (int i=0; i < channel.getNumberOfItems(); i++) {
      paintItem(g, bounds, i);
    }
    showStatus();
  }

  /**
   * paints the background box
   *
   * @param   g Graphics for painting
   * @param   bounds Rectangle to paint inside
   */
  private void paintBox(Graphics g, Rectangle bounds) {
    g.setColor(boxBackground);
    g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
    narrowBounds(bounds, boxBorder, boxBorder);
    g.setColor(boxForeground);
    g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
  }

  /**
   * paints the RSS channel title
   *
   * @param   g Graphics for painting
   * @param   bounds Rectangle to paint inside
   */
  private void paintTitle(Graphics g, Rectangle bounds) {
    paintLine(g, bounds, titleBackground, titleBackgroundOnMouseOver,
      titleForeground, titleForegroundOnMouseOver, channel.getChannelTitle(),
      titleFont, titleAlign, TITLE_IS_CURRENT_ITEM, titleLineHeight, titleLineBase);
  }

  /**
   * paints a channel item
   *
   * @param   g Graphics for painting
   * @param   bounds Rectangle to paint inside
   * @param   item the item to paint
   */
  private void paintItem(Graphics g, Rectangle bounds, int item) {
    paintLine(g, bounds, itemBackground, itemBackgroundOnMouseOver,
      itemForeground, itemForegroundOnMouseOver, channel.getItemTitle(item),
      itemFont, itemAlign, item, itemLineHeight, itemLineBase);
  }

  /**
   * paints a line of text, either the title or an item
   *
   * @param   g Graphics for painting
   * @param   bounds Rectangle to paint inside
   * @param   background the background color
   * @param   backgroundOnMouseOver the highlighted background color
   * @param   foreground the foreground color
   * @param   foregroundOnMouseOver the highlighted foreground color
   * @param   line the text line to paint
   * @param   font the font to paint the text line in
   * @param   align the alignment to paint the text line with
   * @param   item the item number of this line in the channel, to decide about highlighting
   * @param   lineHeight the height of this line as determined by the font
   * @param   lineBase the base line of this line as determined by the font
   */
  private void paintLine(Graphics g, Rectangle bounds, Color background,
    Color backgroundOnMouseOver, Color foreground, Color foregroundOnMouseOver,
    String line, Font font, String align, int item, int lineHeight, int lineBase) {
    g.setColor(currentItem == item ? backgroundOnMouseOver : background);
    g.fillRect(bounds.x, bounds.y, bounds.width, lineHeight);
    g.setColor(currentItem == item? foregroundOnMouseOver : foreground);
    g.setFont(font);
    int textX = bounds.x + 1; int textW = bounds.width - 2;
    String prunedLine = getPrunedString(line, font, textW);
    g.drawString(prunedLine, textX + getStringStartPosX(prunedLine, font, textW, align), bounds.y + lineBase);
    bounds.y += lineHeight + itemMargin;
  }

  /**
   * shows the applet's status
   * @see java.applet.Applet#showStatus
   */
  private void showStatus() {
    String status;
    if (currentItem == NO_CURRENT_ITEM || currentItem >= channel.getNumberOfItems()) status = "";
    else if (currentItem == TITLE_IS_CURRENT_ITEM) status = channel.getChannelLink().toString();
    else status = channel.getItemLink(currentItem).toString();
    getAppletContext().showStatus(status);
  }

  // event handlers

  /**
   * called on mouse button up
   * @see java.applet.Applet#mouseUp
   */
  public boolean mouseUp(Event e, int x, int y) {
    int item = getItemFromCoordinates(x, y);
    if (item == NO_CURRENT_ITEM) return true;
    URL target = (item == TITLE_IS_CURRENT_ITEM) ? channel.getChannelLink() : channel.getItemLink(item);
    getAppletContext().showDocument(target);
    return true;
  }

  /**
   * called on mouse exiting the applet
   * @see java.applet.Applet#mouseExit
   */
  public boolean mouseExit(Event e, int x, int y) {
    Rectangle r = getItemBounds(currentItem);

    currentItem = NO_CURRENT_ITEM;

    repaint(r.x, r.y, r.width, r.height);

    return true;

  }


  /**

   * called on mouse move
   * @see java.applet.Applet#mouseMove
   */
  public boolean mouseMove(Event e, int x, int y) {

    int newItem = getItemFromCoordinates(x, y);

    if (currentItem != newItem) {

      Rectangle r = getItemBounds(currentItem);

      currentItem = newItem;

      repaint(r.x, r.y, r.width, r.height);

      r = getItemBounds(newItem);

      repaint(r.x, r.y, r.width, r.height);

    }

    return true;

  }


  /**
   * gets the underlying item corresponding to the (x,y) coordinates
   *
   * @param   x the horizontal coordinate
   * @param   y the vertical coordinate
   * @return  the item position
   */
  private int getItemFromCoordinates(int x, int y) {
    int item = (y - (titleLineHeight-itemLineHeight) - boxBorder) / (itemLineHeight + itemMargin) - 1;
    return (item > NO_CURRENT_ITEM && item < channel.getNumberOfItems()) ? item : NO_CURRENT_ITEM;
  }


  /**

   * gets the bounding box of the corresponding item
   *
   * @param   item the RSS channel item number
   * @return  the bounding box Rectangle
   */
  private Rectangle getItemBounds(int item) {

    Rectangle r = new Rectangle();

    r.x = boxBorder;

    r.y = (item < 0 ? 0 : titleLineHeight-itemLineHeight + boxBorder + (item+1) * (itemLineHeight + itemMargin));

    r.width = size().width;

    r.height = (item < 0 ? titleLineHeight : itemLineHeight);

    return r;

  }


  /**

   * narrows a bounding box Rectangle
   *
   * @param   bounds the Rectangle to narrow
   * @param   x the number of horizontal pixels
   * @param   y the number of vertical pixels
   */
  private void narrowBounds(Rectangle bounds, int x, int y) {

    bounds.x += x; bounds.width -= 2*x;
    bounds.y += y; bounds.height -= 2*y;
  }

  /**

   * gets the line height of a Font
   *
   * @param   font the Font to measure
   * @return  the line height in pixels
   */
  private int getLineHeight(Font font) {

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);

    return fm.getHeight() + 2;
  }


  /**

   * gets the base line of a Font
   *
   * @param   font the Font to measure
   * @return  the base line in pixels
   */
  private int getLineBase(Font f) {

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(f);

    return fm.getAscent() + 1;
  }


  /**

   * gets a pruned version of a String to fit a certain width
   *
   * @param   line the String to prune
   * @param   font the Font to use
   * @param   width the width to prune to
   * @return  the pruned String
   */
  private String getPrunedString(String line, Font f, int width) {

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(f);

    if (fm.stringWidth(line) < width) return line;

    final String ellipsis = "...";

    width -= fm.stringWidth(ellipsis);

    String str = line;

    while (fm.stringWidth(str) > width) {

      str = str.substring(0, str.length()-1);

    }

    return str + ellipsis;

  }


  /**

   * gets the start pixel of a String with alignment
   *
   * @param   line the String to prune
   * @param   font the Font to use
   * @param   width the width available in pixels
   * @param   align the alignment to use
   * @return  the start position in horizontal pixels
   */
  private int getStringStartPosX(String line, Font font, int width, String align) {

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);

    int blankWidth = width - fm.stringWidth(line);

    if (blankWidth <= 0 ) return 0;

    if (align.equalsIgnoreCase("right")) {

      return blankWidth;

    }

    else if (align.equalsIgnoreCase("center")) {

      return blankWidth / 2;

    }

    else return 0;

  }


}


